@@ -6,8 +6,6 @@
 
 #include <ngx_config.h>
 #include <ngx_core.h>
-#include <ngx_event.h>
-#include <ngx_event_connect.h>
 
 
 /*
@@ -223,8 +221,8 @@ ngx_ptocidr(ngx_str_t *text, void *cidr)
 ngx_int_t
 ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
 {
-    u_char              *p, *host;
-    size_t               len;
+    u_char              *p, *host, *port_start;
+    size_t               len, port_len;
     ngx_int_t            port;
     ngx_uint_t           i;
     struct hostent      *h;
@@ -235,12 +233,10 @@ ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
     len = u->url.len;
     p = u->url.data;
 
-    if (ngx_strncasecmp(p, "unix:", 5) == 0) {
+    if (ngx_strncasecmp(p, (u_char *) "unix:", 5) == 0) {
 
 #if (NGX_HAVE_UNIX_DOMAIN)
 
-        u->type = NGX_PARSE_URL_UNIX;
-
         p += 5;
         len -= 5;
 
@@ -271,8 +267,8 @@ ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
             return NGX_ERROR;
         }
 
-        u->peers = ngx_pcalloc(cf->pool, sizeof(ngx_peers_t));
-        if (u->peers == NULL) {
+        u->addrs = ngx_pcalloc(cf->pool, sizeof(ngx_peer_addr_t));
+        if (u->addrs == NULL) {
             return NGX_ERROR;
         }
 
@@ -281,19 +277,20 @@ ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
             return NGX_ERROR;
         }
 
-        u->peers->number = 1;
+        u->naddrs = 1;
 
         saun->sun_family = AF_UNIX;
         (void) ngx_cpystrn((u_char *) saun->sun_path, p, len + 1);
 
-        u->peers->peer[0].sockaddr = (struct sockaddr *) saun;
-        u->peers->peer[0].socklen = sizeof(struct sockaddr_un);
-        u->peers->peer[0].name.len = len + 5;
-        u->peers->peer[0].name.data = u->url.data;
-        u->peers->peer[0].uri_separator = ":";
+        u->addrs[0].sockaddr = (struct sockaddr *) saun;
+        u->addrs[0].socklen = sizeof(struct sockaddr_un);
+        u->addrs[0].name.len = len + 5;
+        u->addrs[0].name.data = u->url.data;
 
-        u->host_header.len = sizeof("localhost") - 1;
-        u->host_header.data = (u_char *) "localhost";
+        u->host.len = len;
+        u->host.data = p;
+
+        u->unix_socket = 1;
 
         return NGX_OK;
 
@@ -310,20 +307,19 @@ ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
         return NGX_ERROR;
     }
 
-    u->type = NGX_PARSE_URL_INET;
-
     u->host.data = p;
-    u->host_header.len = len;
-    u->host_header.data = p;
+
+    port_start = NULL;
+    port_len = 0;
 
     for (i = 0; i < len; i++) {
 
         if (p[i] == ':') {
-            u->port.data = &p[i + 1];
+            port_start = &p[i + 1];
             u->host.len = i;
 
             if (!u->uri_part) {
-                u->port.len = &p[len] - u->port.data;
+                port_len = len - (i + 1);
                 break;
             }
         }
@@ -331,20 +327,19 @@ ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
         if (p[i] == '/') {
             u->uri.len = len - i;
             u->uri.data = &p[i];
-            u->host_header.len = i;
 
             if (u->host.len == 0) {
                 u->host.len = i;
             }
 
-            if (u->port.data == NULL) {
-                u->default_port = 1;
-                goto port;
+            if (port_start == NULL) {
+                u->no_port = 1;
+                goto no_port;
             }
 
-            u->port.len = &p[i] - u->port.data;
+            port_len = &p[i] - port_start;
 
-            if (u->port.len == 0) {
+            if (port_len == 0) {
                 u->err = "invalid port";
                 return NGX_ERROR;
             }
@@ -353,18 +348,18 @@ ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
         }
     }
 
-    if (u->port.data) {
+    if (port_start) {
 
-        if (u->port.len == 0) {
-            u->port.len = &p[i] - u->port.data;
+        if (port_len == 0) {
+            port_len = &p[i] - port_start;
 
-            if (u->port.len == 0) {
+            if (port_len == 0) {
                 u->err = "invalid port";
                 return NGX_ERROR;
             }
         }
 
-        port = ngx_atoi(u->port.data, u->port.len);
+        port = ngx_atoi(port_start, port_len);
 
         if (port == NGX_ERROR || port < 1 || port > 65536) {
             u->err = "invalid port";
@@ -375,29 +370,28 @@ ngx_parse_url(ngx_conf_t *cf, ngx_url_t *u)
         port = ngx_atoi(p, len);
 
         if (port == NGX_ERROR) {
-            u->default_port = 1;
             u->host.len = len;
+            u->no_port = 1;
 
-            goto port;
+            goto no_port;
         }
 
-        u->port.len = len;
-        u->port.data = p;
         u->wildcard = 1;
     }
 
-    u->portn = (in_port_t) port;
+    u->port = (in_port_t) port;
 
-port:
+no_port:
 
     if (u->listen) {
-        if (u->portn == 0) {
-            if (u->default_portn == 0) {
+
+        if (u->port == 0) {
+            if (u->default_port == 0) {
                 u->err = "no port";
                 return NGX_ERROR;
             }
 
-            u->portn = u->default_portn;
+            u->port = u->default_port;
         }
 
         if (u->host.len == 1 && u->host.data[0] == '*') {
@@ -425,7 +419,7 @@ port:
                     return NGX_ERROR;
                 }
 
-                u->addr.in_addr = *(in_addr_t *)(h->h_addr_list[0]);
+                u->addr.in_addr = *(in_addr_t *) (h->h_addr_list[0]);
             }
 
         } else {
@@ -435,49 +429,25 @@ port:
         return NGX_OK;
     }
 
-    if (u->default_port) {
-
-        if (u->upstream) {
-            return NGX_OK;
-        }
-
-        if (u->default_portn == 0) {
-            u->err = "no port";
-            return NGX_ERROR;
-        }
-
-        u->portn = u->default_portn;
-
-        u->port.data = ngx_palloc(cf->pool, sizeof("65536") - 1);
-        if (u->port.data == NULL) {
-            return NGX_ERROR;
-        }
-
-        u->port.len = ngx_sprintf(u->port.data, "%d", u->portn) - u->port.data;
-
-    } else if (u->portn) {
-        if (u->portn == u->default_portn) {
-            u->default_port = 1;
-        }
-
-    } else {
-        u->err = "no port";
-        return NGX_ERROR;
-    }
-
     if (u->host.len == 0) {
         u->err = "no host";
         return NGX_ERROR;
     }
 
-    u->peers = ngx_inet_resolve_peer(cf, &u->host, u->portn);
+    if (u->no_resolve) {
+        return NGX_OK;
+    }
 
-    if (u->peers == NULL) {
+    if (u->no_port) {
+        u->port = u->default_port;
+    }
+
+    if (u->port == 0) {
+        u->err = "no port";
         return NGX_ERROR;
     }
 
-    if (u->peers == NGX_CONF_ERROR) {
-        u->err = "host not found";
+    if (ngx_inet_resolve_host(cf, u) != NGX_OK) {
         return NGX_ERROR;
     }
 
@@ -485,23 +455,22 @@ port:
 }
 
 
-ngx_peers_t *
-ngx_inet_resolve_peer(ngx_conf_t *cf, ngx_str_t *name, in_port_t port)
+ngx_int_t
+ngx_inet_resolve_host(ngx_conf_t *cf, ngx_url_t *u)
 {
-    u_char              *host;
+    u_char              *p, *host;
     size_t               len;
     in_addr_t            in_addr;
     ngx_uint_t           i;
-    ngx_peers_t         *peers;
     struct hostent      *h;
     struct sockaddr_in  *sin;
 
-    host = ngx_palloc(cf->temp_pool, name->len + 1);
+    host = ngx_palloc(cf->temp_pool, u->host.len + 1);
     if (host == NULL) {
-        return NULL;
+        return NGX_ERROR;
     }
 
-    (void) ngx_cpystrn(host, name->data, name->len + 1);
+    (void) ngx_cpystrn(host, u->host.data, u->host.len + 1);
 
     /* AF_INET only */
 
@@ -511,92 +480,84 @@ ngx_inet_resolve_peer(ngx_conf_t *cf, ngx_str_t *name, in_port_t port)
         h = gethostbyname((char *) host);
 
         if (h == NULL || h->h_addr_list[0] == NULL) {
-            return NGX_CONF_ERROR;
+            u->err = "host not found";
+            return NGX_ERROR;
         }
 
-        for (i = 0; h->h_addr_list[i] != NULL; i++) { /* void */ }
+        if (u->one_addr == 0) {
+            for (i = 0; h->h_addr_list[i] != NULL; i++) { /* void */ }
+
+        } else {
+            i = 1;
+        }
 
         /* MP: ngx_shared_palloc() */
 
-        peers = ngx_pcalloc(cf->pool,
-                            sizeof(ngx_peers_t) + sizeof(ngx_peer_t) * (i - 1));
-        if (peers == NULL) {
-            return NULL;
+        u->addrs = ngx_pcalloc(cf->pool, i * sizeof(ngx_peer_addr_t));
+        if (u->addrs == NULL) {
+            return NGX_ERROR;
         }
 
-        peers->number = i;
+        u->naddrs = i;
 
         for (i = 0; h->h_addr_list[i] != NULL; i++) {
 
             sin = ngx_pcalloc(cf->pool, sizeof(struct sockaddr_in));
             if (sin == NULL) {
-                return NULL;
+                return NGX_ERROR;
             }
 
             sin->sin_family = AF_INET;
-            sin->sin_port = htons(port);
+            sin->sin_port = htons(u->port);
             sin->sin_addr.s_addr = *(in_addr_t *) (h->h_addr_list[i]);
 
-            peers->peer[i].sockaddr = (struct sockaddr *) sin;
-            peers->peer[i].socklen = sizeof(struct sockaddr_in);
+            u->addrs[i].sockaddr = (struct sockaddr *) sin;
+            u->addrs[i].socklen = sizeof(struct sockaddr_in);
 
             len = INET_ADDRSTRLEN - 1 + 1 + sizeof(":65536") - 1;
 
-            peers->peer[i].name.data = ngx_palloc(cf->pool, len);
-            if (peers->peer[i].name.data == NULL) {
-                return NULL;
+            p = ngx_palloc(cf->pool, len);
+            if (p == NULL) {
+                return NGX_ERROR;
             }
 
-            len = ngx_sock_ntop(AF_INET, (struct sockaddr *) sin,
-                                peers->peer[i].name.data, len);
+            len = ngx_sock_ntop(AF_INET, (struct sockaddr *) sin, p, len);
 
-            peers->peer[i].name.len =
-                                    ngx_sprintf(&peers->peer[i].name.data[len],
-                                                ":%d", port)
-                                      - peers->peer[i].name.data;
-
-            peers->peer[i].uri_separator = "";
-
-            peers->peer[i].weight = NGX_CONF_UNSET_UINT;
-            peers->peer[i].max_fails = NGX_CONF_UNSET_UINT;
-            peers->peer[i].fail_timeout = NGX_CONF_UNSET;
+            u->addrs[i].name.len = ngx_sprintf(&p[len], ":%d", u->port) - p;
+            u->addrs[i].name.data = p;
         }
 
     } else {
 
         /* MP: ngx_shared_palloc() */
 
-        peers = ngx_pcalloc(cf->pool, sizeof(ngx_peers_t));
-        if (peers == NULL) {
-            return NULL;
+        u->addrs = ngx_pcalloc(cf->pool, sizeof(ngx_peer_addr_t));
+        if (u->addrs == NULL) {
+            return NGX_ERROR;
         }
 
         sin = ngx_pcalloc(cf->pool, sizeof(struct sockaddr_in));
         if (sin == NULL) {
-            return NULL;
+            return NGX_ERROR;
         }
 
-        peers->number = 1;
+        u->naddrs = 1;
 
         sin->sin_family = AF_INET;
-        sin->sin_port = htons(port);
+        sin->sin_port = htons(u->port);
         sin->sin_addr.s_addr = in_addr;
 
-        peers->peer[0].sockaddr = (struct sockaddr *) sin;
-        peers->peer[0].socklen = sizeof(struct sockaddr_in);
+        u->addrs[0].sockaddr = (struct sockaddr *) sin;
+        u->addrs[0].socklen = sizeof(struct sockaddr_in);
 
-        peers->peer[0].name.data = ngx_palloc(cf->pool,
-                                              name->len + sizeof(":65536") - 1);
-        if (peers->peer[0].name.data == NULL) {
-            return NULL;
+        p = ngx_palloc(cf->pool, u->host.len + sizeof(":65536") - 1);
+        if (p == NULL) {
+            return NGX_ERROR;
         }
 
-        peers->peer[0].name.len = ngx_sprintf(peers->peer[0].name.data, "%V:%d",
-                                              name, port)
-                                  - peers->peer[0].name.data;
-
-        peers->peer[0].uri_separator = "";
+        u->addrs[0].name.len = ngx_sprintf(p, "%V:%d", &u->host, u->port) - p;
+        u->addrs[0].name.data = p;
     }
 
-    return peers;
+    return NGX_OK;
 }
